package shapecalculus.behaviour;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import repast.simphony.random.RandomHelper;
import repast.simphony.space.SpatialException;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.util.SimUtilities;
import shapecalculus.ActionsContext;
import shapecalculus.BasicShape;
import shapecalculus.Utils;
import shapecalculus.boneremodelling.BestDistanceAction;

public class ActionDispatcher {

	private Grid<Object> grid;
	private ActionsContext actContext;
	private List<AtomicBehaviour> nonDistanceActions;

	public ActionDispatcher(ActionsContext actContext) {
		this.actContext = actContext;
		this.grid = this.actContext.getGrid();
		this.nonDistanceActions = new ArrayList<AtomicBehaviour>();
	}

	public void dispatchAction(AtomicBehaviour action) {
		if (action instanceof DistanceSensible) {
			this.actContext.add(action);
			this.grid.moveTo(action,
					Utils.getCoords(action.getShape().getDiscretePosition()));
		} else
			this.nonDistanceActions.add(action);
	}

	public void runActions() {
		DistanceSensible currentAction;
		GridPoint pt;
		// DistanceActionDistribution surroundingActions;
		BasicShape s;

		Iterator<Object> it = this.actContext
				.getObjects(DistanceSensible.class).iterator();
		while (it.hasNext()) {
			currentAction = (DistanceSensible) it.next();
			s = currentAction.getShape();
			if (currentAction.isOutput() && !s.isBehaviourPerformed()) {
				pt = this.grid.getLocation(currentAction);
				// surroundingActions = getNeighbourActions(pt,
				// currentAction.getExtent(), currentAction);
				DistanceSensible otherAction = getBestNeighbour(pt,
						currentAction.getExtent(), currentAction);
				// while ((otherAction = surroundingActions.getNext()) != null)
				// {
				if (otherAction != null) {
					currentAction.setSynchronizingAction(otherAction);
					otherAction.setSynchronizingAction(currentAction);
					BasicShape s2 = otherAction.getShape();
					int res1 = s.executeBehaviour(currentAction);
					int res2 = s2.executeBehaviour(otherAction);
					if (res1 == 1 && res2 == 1) {
						s.setBehaviourPerformed(true);
						s2.setBehaviourPerformed(true);
						it.remove();
						this.actContext.remove(currentAction);
						// break;
					}
					currentAction.setSynchronizingAction(null);
					otherAction.setSynchronizingAction(null);
				}

			}
		}

		SimUtilities
				.shuffle(this.nonDistanceActions, RandomHelper.getUniform());
		for (AtomicBehaviour action : this.nonDistanceActions) {
			s = action.getShape();
			if (action instanceof Delay) {
				s.executeBehaviour(action);
			} else if (!s.isBehaviourPerformed()) {
				int result = s.executeBehaviour(action);
				if (result == 1)
					s.setBehaviourPerformed(true);
			}
		}

	}

	public void reset() {
		this.actContext.clear();
		// for (Object o : this.actContext.getObjects(DistanceSensible.class))
		// this.actContext.remove(o);
		this.nonDistanceActions = new ArrayList<AtomicBehaviour>();
	}

	private DistanceSensible getBestNeighbour(GridPoint pt, GridPoint extent,
			DistanceSensible currentAction) {
		BestDistanceAction retVal = new BestDistanceAction();
		int x = Math.abs(extent.getX());
		int y = Math.abs(extent.getY());
//		int z = Math.abs(extent.getZ());
		int x1 = pt.getX();
		int y1 = pt.getY();
//		int z1 = pt.getZ();
		for (int i = -x; i <= x; i++)
			for (int j = -y; j <= y; j++){
//				for (int k = -z; k <= z; k++) {
					try {
						for (Object o : this.grid.getObjectsAt(x1 + i, y1 + j)) {
							DistanceSensible act = (DistanceSensible) o;
							if (currentAction.isCompatibleWith(act)
									&& !act.getShape().isBehaviourPerformed()) {
								// if(i == 0 && j==0 && currentAction instanceof
								// Bind &&
								// currentAction.getChannel().getChannelName().equals("rank"))
								// System.out.println("p");
								retVal.add(act, Utils
										.vectorIntensity(new NdPoint(i, j)));
							}
						}
					} catch (SpatialException e) {

					}
				}

		return retVal.getBest();
	}

}
