package shapecalculus.behaviour;

import java.util.Iterator;

import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.GridPoint;
import shapecalculus.ShapeException;
import shapecalculus.Surface;
import shapecalculus.behaviour.channel.Channel;
import shapecalculus.behaviour.channel.InChannel;
import shapecalculus.behaviour.channel.OutChannel;

public class Bind extends DistanceSensible {

	public Bind(Channel channel) {
		this.channel = channel;
	}

	public Object clone() {
		Bind b = new Bind(this.channel);
		b.initPath(this.path);
		return b;
	}

	@Override
	public boolean canPerform() {
		return this.channel.getBindedTo() == null
				&& this.channel.hasFreeSurface();
	}

	@Override
	public GridPoint getExtent() {
		if (this.channel instanceof OutChannel)
			return ((OutChannel) this.channel).getExtent();
		return new GridPoint(1, 1, 1);
	}

	@Override
	public boolean isCompatibleWith(DistanceSensible action) {
		return this.channel.canBindWith(action.getChannel())
				&& (action instanceof Bind);
	}

	@Override
	public int perform(Iterator<Integer> path) {
		DistanceSensible otherAct = this.synchronizingAction;
		if (this.isOutput() && otherAct != null) {
			// bind channels
			this.channel.setBindedTo(otherAct.channel);
			otherAct.channel.setBindedTo(this.channel);
			// Network<Object> net =
			// (Network<Object>)this.getShape().getContext().getProjection("bind_network");
			// net.addEdge(this.getShape(), otherAct.getShape());
			
			// move towards other surface
			otherAct.getShape().moveTo(
					this.channel.getSurface().getAdjacentPoint(
							this.getShape().getPosition()), false);
			this.getShape().setVelocity(new NdPoint(0,0,0));
			otherAct.getShape().setVelocity(new NdPoint(0,0,0));
//			this.getShape().moveTo(
//					otherAct.channel.getSurface().getAdjacentPoint(
//							otherAct.getShape().getDiscretePosition()), false);
		}
		return 0;
	}

	@Override
	public Behaviour returnDistanceSensibleEquivalent(int[] directions) {
		Behaviour[] operands = new Bind[directions.length];
		for (int i = 0; i < directions.length; i++) {
			try {
				if (this.channel instanceof InChannel)
					operands[i] = new Bind(new InChannel(
							this.channel.getChannelName(),
							this.channel.getShape(),
							new Surface(directions[i]),
							this.channel.getPriority()));
				else if (this.channel instanceof OutChannel)
					operands[i] = new Bind(new OutChannel(
							this.channel.getChannelName(),
							this.channel.getShape(),
							new Surface(directions[i]),
							((OutChannel) this.channel).getExtent(),
							this.channel.getPriority()));
			} catch (ShapeException e) {
				e.printStackTrace();
			}
		}
		try {
			return new Choice(operands);
		} catch (BehaviourException e) {
			e.printStackTrace();
		}
		return null;
	}
}
