package shapecalculus.behaviour;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

public class Recursion extends NonAtomicBehaviour {

	private Behaviour initialBehaviour;
	private int iterations;

	public Recursion(Behaviour behaviour) {
		this.operands = new ArrayList<Behaviour>();
		if (behaviour instanceof DistanceSensible) {
			DistanceSensible bsens = (DistanceSensible) behaviour;
			int[] directions = bsens.getSurface().getDirection();
			if (directions.length > 1)
				this.operands.add(bsens
						.returnDistanceSensibleEquivalent(directions));
			else
				this.operands.add(behaviour);
		} else
			this.operands.add(behaviour);

		this.initialBehaviour = (Behaviour) this.operands.get(0).clone();
		this.currentIndex = 0;
		this.iterations = Integer.MAX_VALUE;
	}

	public Recursion(Behaviour behaviour, int iterations) {
		this.operands = new ArrayList<Behaviour>();
		if (behaviour instanceof DistanceSensible) {
			DistanceSensible bsens = (DistanceSensible) behaviour;
			int[] directions = bsens.getSurface().getDirection();
			if (directions.length > 1)
				this.operands.add(bsens
						.returnDistanceSensibleEquivalent(directions));
			else
				this.operands.add(behaviour);
		} else
			this.operands.add(behaviour);

		this.initialBehaviour = (Behaviour) this.operands.get(0).clone();
		this.currentIndex = 0;
		this.iterations = iterations;
	}

	private Recursion() {
		this.operands = new ArrayList<Behaviour>();
		this.currentIndex = 0;
	}

	// @Override
	// public int perform() {
	// int ret = this.operands.get(this.currentIndex).perform();
	//
	// if (ret == 0) {
	// this.operands.remove(this.currentIndex);
	// this.operands.add(this.initialBehaviour);
	// this.initialBehaviour = (Behaviour) this.initialBehaviour.clone();
	// ret = 1;
	// // ret = this.behaviour.perform();
	// }
	// return ret;
	// }

	@Override
	public int perform(Iterator<Integer> path) {
		if (iterations <= 0)
			return 0;
		if (!(path.hasNext() && isPathInitialized))
			return -1;
		this.currentIndex = path.next();
		int ret = this.operands.get(this.currentIndex).perform(path);
		iterations--;
		if (ret == 0 && iterations > 0) {
			this.operands.remove(this.currentIndex);
			this.operands.add(this.initialBehaviour);
			this.initialBehaviour = (Behaviour) this.initialBehaviour.clone();
			this.initPath(this.path);
			ret = 1;
			// ret = this.behaviour.perform();
		}
		return ret;
	}

	@Override
	public Set<AtomicBehaviour> getEnabledActions() {
		Set<AtomicBehaviour> ret = new HashSet<AtomicBehaviour>();
		Behaviour behaviour = this.operands.get(currentIndex);
		if (behaviour instanceof AtomicBehaviour
				&& ((AtomicBehaviour) behaviour).canPerform())
			ret.add((AtomicBehaviour) behaviour);

		else if (behaviour instanceof NonAtomicBehaviour)
			ret.addAll(((NonAtomicBehaviour) behaviour).getEnabledActions());
		return ret;

	}

	public Object clone() {
		Recursion r = new Recursion();
		r.operands.add((Behaviour) this.operands.get(currentIndex).clone());
		r.initialBehaviour = (Behaviour) this.initialBehaviour.clone();
		r.initPath(this.path);
		r.iterations = this.iterations;
		return r;
	}

}
