package shapecalculus.behaviour;

import java.util.Iterator;

import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.GridPoint;
import shapecalculus.BasicShape;
import shapecalculus.ShapeException;
import shapecalculus.Surface;
import shapecalculus.behaviour.channel.Channel;
import shapecalculus.behaviour.channel.InChannel;
import shapecalculus.behaviour.channel.OutChannel;

public class Split extends DistanceSensible {

	public Split(Channel c) {
		this.channel = c;
	}

	@Override
	public int perform(Iterator<Integer> path) {
		DistanceSensible otherAct = this.synchronizingAction;
		if (this.isOutput() && otherAct != null) {
			// unbind channels
			Channel outChan = this.channel.getBindedEquivalent();
			Channel inChan = otherAct.channel.getBindedEquivalent();
			if (inChan != null && outChan != null) {
				this.channel.getBindedEquivalent().removeBond();
				otherAct.channel.getBindedEquivalent().removeBond();
			}
			BasicShape shape1 = this.getShape();
			BasicShape shape2 = otherAct.getShape();
//			NdPoint[] newVels = Utils.calculateExplosionVel(shape1.getMass(),
//					shape2.getMass(), shape1.getVelocity());
//			shape1.setVelocity(newVels[0]);
//			shape2.setVelocity(newVels[1]);
			shape1.setVelocity(new NdPoint(0,0,0));
			shape2.setVelocity(new NdPoint(0,0,0));
		}
		return 0;
	}

	@Override
	public GridPoint getExtent() {
		return new GridPoint(1, 1, 1);
	}

	@Override
	public boolean isCompatibleWith(DistanceSensible action) {
		return this.channel.canSplitFrom(action.getChannel())
				&& (action instanceof Split);
	}

	@Override
	public boolean canPerform() {
		Channel bindedChan = this.channel.getBindedEquivalent();
		return bindedChan != null
				&& this.channel.getChannelName() == bindedChan.getChannelName();
	}

	@Override
	public Object clone() {
		Split s = new Split(this.channel);
		s.initPath(this.path);
		return s;
	}

	@Override
	public Behaviour returnDistanceSensibleEquivalent(int[] directions) {
		Behaviour[] operands = new Split[directions.length];
		for (int i = 0; i < directions.length; i++) {
			try {
				if (this.channel instanceof InChannel)
					operands[i] = new Split(new InChannel(
							this.channel.getChannelName(),
							this.channel.getShape(),
							new Surface(directions[i]),
							this.channel.getPriority()));
				else if (this.channel instanceof OutChannel)
					operands[i] = new Split(new OutChannel(
							this.channel.getChannelName(),
							this.channel.getShape(),
							new Surface(directions[i]),
							((OutChannel) this.channel).getExtent(),
							this.channel.getPriority()));
			} catch (ShapeException e) {
				e.printStackTrace();
			}
		}
		try {
			return new Choice(operands);
		} catch (BehaviourException e) {
			e.printStackTrace();
		}
		return null;
	}

}
