package shapecalculus.boneremodelling;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import repast.simphony.context.Context;
import repast.simphony.context.DefaultContext;
import repast.simphony.context.space.grid.GridFactory;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.random.DefaultRandomRegistry;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.StrictBorders;
import repast.simphony.valueLayer.ContinuousValueLayer;
import repast.simphony.valueLayer.GridValueLayer;
import shapecalculus.ActionsContext;
import shapecalculus.BasicShape;
import shapecalculus.ComplexShape;
import shapecalculus.behaviour.DispatcherFactory;

public class BRContext extends DefaultContext<Object> implements
		ContextBuilder<Object> {

	protected static Fracture fracture;
	protected static MinAttraction obAttraction;
	protected static Diffusion ranklField;
	public static int initialOyNum;
	public static int currentOyNum;
	protected static double boneAbsorbed;
	protected static double boneFormed;
	private static int totalBone;
	protected static DefaultRandomRegistry reg;
	protected static int currentObNum;
	protected static int currentOcNum;
	private static RANKLDistribution ranklDist;
	private static double cumulativeFormation;
	private static double cumulativeResorption;
	private static double boneMass;

	@Override
	public Context build(Context<Object> context) {
		context.setId("shapecalculus");
		currentObNum = 0;
		currentOcNum = 0;
		currentOyNum = 0;
		cumulativeFormation = 0;
		cumulativeResorption = 0;
		boneMass = 0;
		// create default exponential distribution for actions
		reg = new DefaultRandomRegistry();
		reg.createExponential(1);
		BRContext.ranklDist = new RANKLDistribution();
		int[] bmusize = ParametersManager.getBMUSize();
		BRContext.fracture = new Fracture();
//		ContinuousSpaceFactory spaceFactory = ContinuousSpaceFactoryFinder
//				.createContinuousSpaceFactory(null);


		GridFactory gridFactory = GridFactoryFinder.createGridFactory(null);
		Grid<Object> grid = gridFactory.createGrid("grid", context,
				GridBuilderParameters.multiOccupancyND(
						new BMUGridAdder<Object>(null), new StrictBorders(),
						bmusize));

		ActionsContext actContext = new ActionsContext(new StrictBorders(),
				bmusize);

		context.addSubContext(actContext);
		new DispatcherFactory(actContext);

		totalBone = bmusize[0] * bmusize[1] * 100;
		// add value layer(s)
		GridValueLayer vlbone = new GridValueLayer("BoneMass", 100, true,
				new StrictBorders(), bmusize[0], bmusize[1]);
		context.addValueLayer(vlbone);

		initBoneValueLayer(vlbone);
		// for (int i = 0; i < bmusize[0]; i++)
		// for (int j = 0; j < bmusize[1]; j++)
		// context.add(new BoneNode(context, i, j));

		GridValueLayer vloc = new GridValueLayer("OcVelocity", 0, true,
				new StrictBorders(), bmusize[0], bmusize[1]);
		context.addValueLayer(vloc);
		GridValueLayer vlrankl = new GridValueLayer("RANKL", 0, true,
				new StrictBorders(), bmusize[0], bmusize[1]);
		context.addValueLayer(vlrankl);

		ContinuousValueLayer vlob = new ContinuousValueLayer("ObDensity", 0,
				true, new repast.simphony.space.continuous.StrictBorders(),
				bmusize[0], bmusize[1]);
		context.addValueLayer(vlob);

		ContinuousValueLayer vlocs = new ContinuousValueLayer("OcDensity", 0,
				true, new repast.simphony.space.continuous.StrictBorders(),
				bmusize[0], bmusize[1]);
		context.addValueLayer(vlocs);

		BRContext.obAttraction = new MinAttraction();

		initialOyNum = BRContext.fracture.getPoints().size();
		currentOyNum = initialOyNum;

		return context;
	}

	private void initBoneValueLayer(GridValueLayer vlbone) {
		String path = ParametersManager.getDensityPath();
		double width=vlbone.getDimensions().getWidth();
		double height=vlbone.getDimensions().getHeight();
		if (path.equals("false")) {
			String boneType=ParametersManager.getBoneType();
			
			for (int i = 0; i < width; i++)
				for (int j = 0; j < height; j++) {
					double val = 100 - 100 * BRContext.reg.getExponential()
							.nextDouble(25);
					vlbone.set(val, i, j);
				} 
			if(boneType.equals("trabecular")){
				for (int i = 0; i < width; i++)
					for (int j = 0; j < height; j++){
						if((Math.pow(i,2)+Math.pow(j-height/2,2))<=225)
							vlbone.set(0, i, j);
						else if((Math.pow(i-width,2)+Math.pow(j-height/2,2))<=225)
							vlbone.set(0, i, j);
						else if((Math.pow(i-width/2,2)+Math.pow(j+70,2))<=10000)
							vlbone.set(0, i, j);
						else if((Math.pow(i-width/2,2)+Math.pow(j-height-70,2))<=10000)
							vlbone.set(0, i, j);
					}
				}
		} else {
			try {
				LineNumberReader r = new LineNumberReader(new FileReader(path));
				r.readLine();
				r.readLine();
				String line;
				String[] fields;
				while ((line = r.readLine()) != null) {
					fields = line.split("\t");
					vlbone.set(Double.parseDouble(fields[2]),
							Integer.parseInt(fields[0]),
							Integer.parseInt(fields[1]));
				}

				r.close();
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@ScheduledMethod(priority = ScheduleParameters.FIRST_PRIORITY, start = 1)
	public void insertOsteocytes() {
		BRContext.ranklField = new Diffusion(
				(GridValueLayer) getValueLayer("OcVelocity"));
		Osteocyte oy;
		for (int i = 0; i < BRContext.fracture.getPoints().size(); i++) {
			oy = new Osteocyte(1, this,
					(Grid<Object>) this.getProjection("grid"),
					/*(ContinuousSpace<Object>) this.getProjection("space")*/null, null);
			this.add(oy);
			oy.setVelocity(new NdPoint(0, 0, 0));
		}
	}

	@ScheduledMethod(start = 2, interval = 1)
	public void resetOsteocytes() {
		// System.out.println("oy=" + currentOyNum + "; ocKill=" + numOckiller);
		if (BRContext.obAttraction.isEmpty() && currentOyNum == 0
				&& getBoneFormed() == 0) {
			try {
				PrintWriter pw = new PrintWriter(
						"C:\\Users\\nicopao\\Desktop\\br simulation data\\bonMazz"
								+ System.currentTimeMillis() + ".txt");
				pw.println("Tick: "
						+ RunEnvironment.getInstance().getCurrentSchedule()
								.getTickCount());

				pw.println("X\tY\tValue");
				GridValueLayer bonemass = (GridValueLayer) this
						.getValueLayer("BoneMass");
				for (int i = 0; i < bonemass.getDimensions().getWidth(); i++)
					for (int j = 0; j < bonemass.getDimensions().getHeight(); j++) {
						pw.println(i + "\t" + j + "\t" + bonemass.get(i, j));
					}
				pw.close();
				DispatcherFactory.getActionsContext().clear();
				this.clear();
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			RunEnvironment.getInstance().endRun();

			/*
			 * ArrayList<Object> toDel = new ArrayList<Object>(); for (Object o
			 * : getObjects(Osteoclast.class)) toDel.add(o); for (Object o :
			 * getObjects(Osteoblast.class)) toDel.add(o); for (Object o :
			 * getObjects(OsteoclastKiller.class)) toDel.add(o); for (Object o :
			 * getObjects(OsteoblastKiller.class)) toDel.add(o); for (Object o :
			 * getObjects(MatureOsteoblast.class)) toDel.add(o); for (Object o :
			 * getObjects(OsteoblastProducer.class)) toDel.add(o); for (Object o
			 * : toDel) this.remove(o);
			 * 
			 * BRContext.fracture = new Fracture(); Osteocyte oy; for (int i =
			 * 0; i < BRContext.fracture.getPoints().size(); i++) { oy = new
			 * Osteocyte(1, this, (Grid<Object>) this.getProjection("grid"),
			 * null); this.add(oy); oy.setVelocity(new NdPoint(0, 0, 0)); }
			 */
		}
	}

	public double getFlexure(double n) {
		// double d1 = (n + 1) / 20;
		// d1 = d1 * Math.random();
		// int ret = (int) (d1 - n / 40);
		// return ret;
		return RandomHelper.getUniform().nextDoubleFromTo(-n / 20, n / 20);
	}

	public void resetOcLayer() {
		ContinuousValueLayer vl = (ContinuousValueLayer) this
				.getValueLayer("OcDensity");
		for (int i = 0; i < vl.getDimensions().getWidth(); i++)
			for (int j = 0; j < vl.getDimensions().getHeight(); j++)
				vl.set(0, i, j);
	}

	public void resetObLayer() {
		ContinuousValueLayer vl = (ContinuousValueLayer) this
				.getValueLayer("ObDensity");
		for (int i = 0; i < vl.getDimensions().getWidth(); i++)
			for (int j = 0; j < vl.getDimensions().getHeight(); j++)
				vl.set(0, i, j);
	}

	@ScheduledMethod(priority = ScheduleParameters.FIRST_PRIORITY, start = 2, interval = 1)
	public void updateOb() {
		if (getOsteocyteCount() < initialOyNum / 2) {
			int[] xmax = ParametersManager.getMaximumCellNum();
			resetObLayer();
			int x2 = (int) ((xmax[1] - getOsteoblastCount() + getFlexure(xmax[1])) / ParametersManager
					.getAgingFactor());
			// if (x1 == 0 && x2 == 0) {
			// x1new = (int) ParametersManager.getSteadyState()[0];
			// x2new = (int) ParametersManager.getSteadyState()[1];
			// } else {
			// double alpha1 = ParametersManager.getFormationRates()[0];
			// double alpha2 = ParametersManager.getFormationRates()[1];
			// double[] autocrine = ParametersManager.getAutocrine();
			// double[] paracrine = ParametersManager.getParacrine();
			// x1new = (int) (x1 + alpha1 * Math.pow(x1, autocrine[0])
			// * Math.pow(x2, paracrine[1]));
			// x2new = (int) (x2 + alpha2 * Math.pow(x2, autocrine[1])
			// * Math.pow(x1, paracrine[0]));
			// }
			Osteoblast ob;
			for (int i = 0; i < x2; i++) {
				ob = new Osteoblast(1, this,
						(Grid<Object>) this.getProjection("grid"),
						/*(ContinuousSpace<Object>) this.getProjection("space")*/null,
						null);
				this.add(ob);
				ob.setVelocity(new NdPoint(0, 0, 0));
				ob.setMotion(new ObMotion());
			}
		}
	}

	@ScheduledMethod(priority = ScheduleParameters.FIRST_PRIORITY, start = 1, interval = 1)
	public void updateOc() {
		if (getOsteocyteCount() > 0) {
			int[] xmax = ParametersManager.getMaximumCellNum();
			resetOcLayer();
			double currMaxOc = xmax[0]
					* (1 + BRContext.ranklDist.getNextDouble(ranklField
							.getAvg()));
			int x1 = (int) (currMaxOc - getOsteoclastCount() + getFlexure(currMaxOc));
			// if (x1 == 0 && x2 == 0) {
			// x1new = (int) ParametersManager.getSteadyState()[0];
			// x2new = (int) ParametersManager.getSteadyState()[1];
			// } else {
			// double alpha1 = ParametersManager.getFormationRates()[0];
			// double alpha2 = ParametersManager.getFormationRates()[1];
			// double[] autocrine = ParametersManager.getAutocrine();
			// double[] paracrine = ParametersManager.getParacrine();
			// x1new = (int) (x1 + alpha1 * Math.pow(x1, autocrine[0])
			// * Math.pow(x2, paracrine[1]));
			// x2new = (int) (x2 + alpha2 * Math.pow(x2, autocrine[1])
			// * Math.pow(x1, paracrine[0]));
			// }
			Osteoclast oc;
			for (int i = 0; i < x1; i++) {
				oc = new Osteoclast(1, this,
						(Grid<Object>) this.getProjection("grid"),
						/*(ContinuousSpace<Object>) this.getProjection("space")*/null,
						null);
				this.add(oc);
				oc.setVelocity(new NdPoint(0, 0, 0));
				oc.setMotion(new OcMotion());
			}
		}
	}

	@ScheduledMethod(/* priority = ScheduleParameters.FIRST_PRIORITY, */start = 1, interval = 1)
	public void updateContext() {
		boneMass=0;
		BRActionDispatcher dispatcher = DispatcherFactory.getBRDispatcher();

		List<BasicShape> shapesList = new ArrayList<BasicShape>();
		for (Object o : getObjects(BasicShape.class)) {
			BasicShape s = (BasicShape) o;
			s.setBehaviourPerformed(false);
			s.setComplexShape(null);
			shapesList.add(s);
		}
		List<BasicShape> currentComplex;
		Set<ComplexShape> complexSet = new HashSet<ComplexShape>();
		BasicShape s;
		List<BasicShape> shapesList1 = new ArrayList<BasicShape>();
		shapesList1.addAll(shapesList);
		while (!shapesList.isEmpty()) {
			s = shapesList.get(0);
			currentComplex = new ArrayList<BasicShape>();
			currentComplex = s.buildComplexShape(currentComplex);
			if (currentComplex.size() == 1)
				shapesList.remove(s);
			else {
				complexSet.add(new ComplexShape(currentComplex));
				shapesList.removeAll(currentComplex);
			}
		}
		for (BasicShape shape : shapesList1)
			shape.updateVelocity();
		for (ComplexShape shape : complexSet)
			shape.updateVelocity();
		BRContext.ranklField.copyValues(this);
		dispatcher.reset();
		resetAttractionVectors();

		for (BasicShape shape : shapesList1)
			shape.dispatchEnabledActions(dispatcher);

		dispatcher.runActions();
		cumulativeFormation+=getBoneFormed();
		cumulativeResorption+=getBoneAbsorbed();
		boneMass=boneMass+cumulativeFormation-cumulativeResorption;
	}

	public void resetAttractionVectors() {
		BRContext.obAttraction.reset();
		BRContext.ranklField.reset();
		BRContext.boneAbsorbed = 0;
		BRContext.boneFormed = 0;
	}

	public int getActOsteoblastCount() {
		int i = 0;
		Osteoblast ob;
		for (Object o : getObjects(Osteoblast.class)) {
			ob = (Osteoblast) o;
			if (ob.isActive())
				i++;
		}
		return i;
	}

	public int getActOsteoclastCount() {
		int i = 0;
		Osteoclast oc;
		for (Object o : getObjects(Osteoclast.class)) {
			oc = (Osteoclast) o;
			if (oc.isActive())
				i++;
		}
		currentOcNum = i;
		return i;
	}

	public int getOsteoblastCount() {
		int i = 0;
		Osteoblast ob;
		for (Object o : getObjects(Osteoblast.class)) {
			ContinuousValueLayer vlob = (ContinuousValueLayer) this
					.getValueLayer("ObDensity");
			ob = (Osteoblast) o;
			NdPoint pos = ob.getPosition();
			double val = vlob.get(pos.getX(), pos.getY());
			vlob.set(val + 1, pos.getX(), pos.getY());
			i++;
		}
		currentObNum = i;
		return i;
	}

	public void removeOsteoclast() {

	}

	public int getOsteoclastCount() {
		int i = 0;
		Osteoclast oc;
		for (Object o : getObjects(Osteoclast.class)) {
			ContinuousValueLayer vloc = (ContinuousValueLayer) this
					.getValueLayer("OcDensity");
			oc = (Osteoclast) o;
			NdPoint pos = oc.getPosition();
			double val = vloc.get(pos.getX(), pos.getY());
			vloc.set(val + 1, pos.getX(), pos.getY());
			i++;
		}
		currentOcNum = i;
		return i;
	}

	public int getOsteocyteCount() {
		int i = 0;
		for (Object o : getObjects(Osteocyte.class))
			i++;
		currentOyNum = i;
		return i;
	}

	public void manageSpatialException(Object o) {
		this.remove(o);
	}

	public double getBoneAbsorbed() {
		//96 normalizes bone
		return BRContext.boneAbsorbed/96;
	}

	public double getBoneFormed() {
		return BRContext.boneFormed/96;

	}

	public double getTotalBone() {
		return BRContext.totalBone;

	}

	public double getRANKLmax() {
		return ranklField.getMax();
	}

	public double getRANKLavg() {
		// 200 normalizes rankl
		return ranklField.getAvg()/200;
	}

	public double getRANKLmin() {
		return ranklField.getMin();
	}

	public static double getCumulativeFormation() {
		return cumulativeFormation;
	}

	public static double getCumulativeResorption() {
		return cumulativeResorption;
	}

	public static double getBoneMass() {
		return boneMass;
	}
	
	
}
