package shapecalculus.boneremodelling;

import java.util.HashMap;
import java.util.Map;

import repast.simphony.context.Context;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.valueLayer.GridValueLayer;

public class Diffusion {

	private GridValueLayer valueLayer;
	private Map<GridPoint, Double> rawValues;
	private boolean init;
	private double avg;
	private double max;
	private double min;
	private double totalCells;

	public Diffusion(GridValueLayer valueLayer) {
		this.valueLayer = valueLayer;
		rawValues = new HashMap<GridPoint, Double>();
		avg = 0.1;
		this.max=0;
		this.min=Double.MAX_VALUE;
		totalCells = this.valueLayer.getDimensions().getWidth()
				* this.valueLayer.getDimensions().getHeight();
	}

	public double getMax() {
		return max;
	}

	public double getMin() {
		return min;
	}

	public void add(GridPoint point, double rate) {
		Double curr = this.rawValues.get(point);
		rate = Math.sqrt(Math.abs(rate)) * (rate < 0 ? -1 : 1);
		if (curr == null)
			// this.rawValues.put(point, Math.sqrt(rate));
			this.rawValues.put(point, rate);
		else
			// this.rawValues.put(point, curr + Math.sqrt(rate));
			this.rawValues.put(point, curr + rate);
		// int x = point.getX();
		// int y = point.getY();
		// double distance;
		// double increment;
		// for (int i = 0; i < this.valueLayer.getDimensions().getWidth(); i+=2)
		// for (int j = 0; j < this.valueLayer.getDimensions().getHeight();
		// j+=2) {
		// double value = this.valueLayer.get(i, j);
		// distance = Math.sqrt(Math.pow(x - i, 2) + Math.pow(y - j, 2));
		// increment=Math.sqrt(rate / (distance + 1));
		// this.valueLayer.set(value + increment, i, j);
		// this.valueLayer.set(value + increment, i+1, j);
		// this.valueLayer.set(value + increment, i, j+1);
		// this.valueLayer.set(value + increment, i+1, j+1);
		// }

	}

	public NdPoint getValue(int... coords) {
		if (!init) {
			for (GridPoint point : this.rawValues.keySet()) {
				int x = point.getX();
				int y = point.getY();
				double distance;
				double increment;
				for (int i = 0; i < this.valueLayer.getDimensions().getWidth(); i++)
					for (int j = 0; j < this.valueLayer.getDimensions()
							.getHeight(); j++) {
						double value = this.valueLayer.get(i, j);
						distance = Math.sqrt(Math.pow(x - i, 2)
								+ Math.pow(y - j, 2));
						increment = this.rawValues.get(point)
								/ Math.sqrt((distance + 1));
						this.valueLayer.set(value + increment, i, j);
						// this.valueLayer.set(value + increment, i + 1, j);
						// this.valueLayer.set(value + increment, i, j + 1);
						// this.valueLayer.set(value + increment, i + 1, j + 1);
					}
			}
		}
		int x = coords[0];
		int y = coords[1];
		double vel = this.valueLayer.get(x, y);
		double velx;
		double vely;
		double width = this.valueLayer.getDimensions().getWidth() - 1;
		double height = this.valueLayer.getDimensions().getHeight() - 1;
		if (x + 2 <= width)
			velx = vel - this.valueLayer.get(x + 2, y);
		else
			velx = this.valueLayer.get(width - 2, y)
					- this.valueLayer.get(width, y);
		if (y + 2 <= height)
			vely = vel - this.valueLayer.get(x, y + 2);
		else
			vely = this.valueLayer.get(x, height - 2)
					- this.valueLayer.get(x, height);
		double rand = ParametersManager.getMotionRandomness()[0];
		// double
		// probx=BRContext.reg.getExponential().nextDouble(Math.sqrt(Math.abs(velx)+1));
		// System.out.println(probx);
		// double
		// proby=BRContext.reg.getExponential().nextDouble(Math.sqrt(Math.abs(vely)+1));
		// System.out.println(proby);
		// return new NdPoint(
		// -velx + RandomHelper.getUniform().nextDoubleFromTo(-probx, probx),
		// -vely + RandomHelper.getUniform().nextDoubleFromTo(-proby, proby),
		// 0);
		return new NdPoint(
				-velx + RandomHelper.getUniform().nextDoubleFromTo(-rand, rand),
				-vely + RandomHelper.getUniform().nextDoubleFromTo(-rand, rand),
				0);
	}

	public void copyValues(Context context) {
		double currValue;
		double sum = 0;
		this.max=0;
		this.min=Double.MAX_VALUE;
		GridValueLayer vl = (GridValueLayer) context.getValueLayer("RANKL");
		for (int i = 0; i < this.valueLayer.getDimensions().getWidth(); i++) {
			for (int j = 0; j < this.valueLayer.getDimensions().getHeight(); j++) {
				currValue = this.valueLayer.get(i, j);
				this.max=Math.max(this.max, currValue);
				this.min=Math.min(this.min, currValue);
				sum += currValue;
				vl.set(currValue, i, j);
			}
		}
		this.avg = sum / totalCells;
//		System.out.println("Snap:"+this.snapshot);
	}

	public double getAvg() {
		return avg;
	}

	public void reset() {
		init = false;
		rawValues = new HashMap<GridPoint, Double>();
		for (int i = 0; i < this.valueLayer.getDimensions().getWidth(); i++)
			for (int j = 0; j < this.valueLayer.getDimensions().getHeight(); j++)
				this.valueLayer.set(0, i, j);

	}
}
