package shapecalculus.boneremodelling;

import java.util.ArrayList;
import java.util.List;

//import org.apache.batik.ext.awt.image.rendered.BumpMap;
//import org.apache.commons.lang.math.RandomUtils;
//import repast.simphony.random.DefaultRandomRegistry;
import repast.simphony.space.grid.GridPoint;

public class Fracture {
	private List<GridPoint> points;
	private List<Boolean> occupied;

	public Fracture() {
		points = new ArrayList<GridPoint>();
		occupied = new ArrayList<Boolean>();
		createFracture();
	}

	private void createFracture() {
		String fractureType = ParametersManager.getFractureType();
		int[] bmusize = ParametersManager.getBMUSize();
		int numOy = bmusize[0] / 3;
		int middle = bmusize[1] / 2;
		numOy = (int) (numOy + Math.random() * numOy / 10 - numOy / 20);
		middle = (int) (middle + Math.random() * middle / 10 - middle / 20);

		if (fractureType.equals("horizontal")) {
			for (int i = 0; i < numOy; i++) {
				points.add(new GridPoint(numOy + i, middle));
				occupied.add(false);
			}

		} else if (fractureType.equals("vertical")) {
			int starty = (bmusize[1] - numOy) / 2;
			int startx = bmusize[0] / 2;
			startx = (int) (startx + Math.random() * startx / 10 - startx / 20);
			for (int i = 0; i < numOy; i++) {
				points.add(new GridPoint(startx, starty + i));
				occupied.add(false);
			}

		} else if (fractureType.equals("zigzagH")) {
			int i = 0;
			int middle1 = middle - numOy / 6;
			middle1 = (int) (middle1 + Math.random() * middle1 / 10 - middle1 / 20);
			while (i < numOy / 3) {
				points.add(new GridPoint(numOy + i, middle1));
				occupied.add(false);
				i++;
			}
			while (i < 2 * numOy / 3) {
				points.add(new GridPoint(numOy + i, middle1 + i - numOy / 3));
				occupied.add(false);
				i++;
			}
			while (i < numOy) {
				points.add(new GridPoint(numOy + i, middle1 + numOy / 3));
				occupied.add(false);
				i++;
			}
		} else if (fractureType.equals("zigzagV")) {
			int i = 0;
			int starty = (bmusize[1] - numOy) / 2;
			int startx = bmusize[0] / 2;
			startx = (int) (startx + Math.random() * startx / 10 - startx / 20);
			int middle1 = startx - numOy / 6;
			middle1 = (int) (middle1 + Math.random() * middle1 / 10 - middle1 / 20);
			while (i < numOy / 3) {
				points.add(new GridPoint(middle1, starty + i));
				occupied.add(false);
				i++;
			}
			while (i < 2 * numOy / 3) {
				points.add(new GridPoint(middle1 + i - numOy / 3, starty + i));
				occupied.add(false);
				i++;
			}
			while (i < numOy) {
				points.add(new GridPoint(middle1 + numOy / 3, starty + i));
				occupied.add(false);
				i++;
			}

		} else if (fractureType.equals("randomCurve")) {
			int currY = middle;
			BRContext.reg.createNormal(bmusize[1] / 2, 1+Math.random()*2);
			for (int i = 0; i < numOy; i++) {
				points.add(new GridPoint(numOy + i, currY));
				occupied.add(false);
				currY=(int) BRContext.reg.getNormal().apply(currY);
			}
		}
	}

	
	public List<GridPoint> getPoints() {
		return points;
	}

	public boolean getOccupied(GridPoint point) {
		return occupied.get(points.indexOf(point));
	}

	public GridPoint getNextFreePoint() {
		GridPoint ret = null;
		for (int i = 0; i < points.size(); i++)
			if (!occupied.get(i)) {
				ret = points.get(i);
				break;
			}
		return ret;
	}

	public void setOccupied(GridPoint point, boolean occupation) {
		occupied.set(points.indexOf(point), occupation);
	}

	// public void setOccupied(int index, boolean occupation) {
	// occupied.set(index, occupation);
	// }
}