package shapecalculus.boneremodelling;

import repast.simphony.context.Context;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;
import shapecalculus.ShapeException;
import shapecalculus.Surface;
import shapecalculus.behaviour.Behaviour;
import shapecalculus.behaviour.BehaviourException;
import shapecalculus.behaviour.Bind;
import shapecalculus.behaviour.Choice;
import shapecalculus.behaviour.Sequence;
import shapecalculus.behaviour.Split;
import shapecalculus.behaviour.channel.InChannel;
import shapecalculus.behaviour.channel.OutChannel;

public class Osteoblast extends Cell {

	private OsteoblastKiller killer;
	private MatureOsteoblast mature;
	private OsteoblastProducer consumer;

	public Osteoblast(double mass, Context context, Grid<Object> grid,
			ContinuousSpace<Object> space, Behaviour behaviour) {
		super(mass, context, grid, space, behaviour);
		this.mature = new MatureOsteoblast(1, context, grid, space, null);
		context.add(this.mature);
		this.consumer = new OsteoblastProducer(mass, context, grid, space,
				behaviour);
		context.add(this.consumer);
		this.killer = new OsteoblastKiller(1, context, grid, space, null, this,
				this.mature, this.consumer);
		context.add(this.killer);
		try {
			Behaviour[] bs = new Sequence[4];
			for (int i = 0; i < 4; i++)
				bs[i] = new Sequence(new Bind(new InChannel("ob", this,
						new Surface(i), 1)), new SetVelocity(this, new NdPoint(
						0, 0, 0)), new Split(new InChannel("ob", this,
						new Surface(i), 1)), new SetVelocity(this, new NdPoint(
						0, 0, 0)));
			Behaviour[] bs1 = new Sequence[4];
			int sens = ParametersManager.getOPGSensibility();
			for (int i = 0; i < 4; i++)
				bs1[i] = new Sequence(new Bind(new OutChannel("rank", this,
						new Surface(i), new GridPoint(sens, sens, 1), 0.5)),
						new Split(new OutChannel("rank", this, new Surface(i),
								new GridPoint(1, 1, 1), 1)));

			this.setBehaviour(new Sequence(new Choice(bs), new ActivateKiller(
					this, this.consumer, this.mature, this.killer), new Choice(
					bs1)));
		} catch (ShapeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (BehaviourException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Override
	public void updateVelocity() {
//		this.killer.setPosition(this.getPosition());
//		this.killer.setDiscretePosition(this.getDiscretePosition());
		this.mature.setPosition(this.getPosition());
		this.mature.setDiscretePosition(this.getDiscretePosition());
		this.consumer.setPosition(this.getPosition());
		this.consumer.setDiscretePosition(this.getDiscretePosition());
		super.updateVelocity();
//		this.killer.setVelocity(this.getVelocity());
//		this.killer.setDiscreteVelocity(this.getDiscreteVelocity());
//		this.killer.updateVelocity();
		this.consumer.setVelocity(this.getVelocity());
		this.consumer.setDiscreteVelocity(this.getDiscreteVelocity());
		this.consumer.updateVelocity();
		this.mature.setVelocity(this.getVelocity());
		this.mature.setDiscreteVelocity(this.getDiscreteVelocity());
		this.mature.updateVelocity();

	}

	// @Override
	// public boolean moveTo(GridPoint newPosition, boolean ignoreComplexShape)
	// {
	// return super.moveTo(newPosition, ignoreComplexShape)
	// /*
	// * && this.killer.moveTo(newPosition, ignoreComplexShape) &&
	// * this.mature.moveTo(newPosition, ignoreComplexShape) &&
	// * this.consumer.moveTo(newPosition, ignoreComplexShape)
	// */;
	// }

	@Override
	public void setDiscretePosition(GridPoint discretePosition) {
		super.setDiscretePosition(discretePosition);
//		this.killer.setDiscretePosition(discretePosition);
		this.mature.setDiscretePosition(discretePosition);
		this.consumer.setDiscretePosition(discretePosition);
	}

	@Override
	public void setPosition(NdPoint position) {
		super.setPosition(position);
//		this.killer.setPosition(position);
		this.mature.setPosition(position);
		this.consumer.setPosition(position);
	}

	@Override
	public void setDiscreteVelocity(GridPoint discreteVelocity) {
		super.setDiscreteVelocity(discreteVelocity);
//		this.killer.setDiscreteVelocity(discreteVelocity);
		this.mature.setDiscreteVelocity(discreteVelocity);
		this.consumer.setDiscreteVelocity(discreteVelocity);
	}

	@Override
	public void setVelocity(NdPoint velocity) {
		super.setVelocity(velocity);
//		this.killer.setVelocity(velocity);
		this.mature.setVelocity(velocity);
		this.consumer.setVelocity(velocity);
	}

	@Override
	public int isOsteoblast() {
		return 1;
	}

	public boolean isActive() {
		return this.consumer.getBehaviour() != null;
	}

}
