package shapecalculus.boneremodelling;

import repast.simphony.context.Context;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;
import shapecalculus.ShapeException;
import shapecalculus.Surface;
import shapecalculus.Utils;
import shapecalculus.behaviour.Behaviour;
import shapecalculus.behaviour.BehaviourException;
import shapecalculus.behaviour.Bind;
import shapecalculus.behaviour.Choice;
import shapecalculus.behaviour.Recursion;
import shapecalculus.behaviour.Sequence;
import shapecalculus.behaviour.Split;
import shapecalculus.behaviour.channel.InChannel;

public class Osteoclast extends Cell {

	private OsteoclastKiller[] killers;
	private boolean hasreached;
	private GridPoint[] discPositions;
	private NdPoint[] positions;
	private boolean isActive;

	public Osteoclast(double mass, Context context, Grid<Object> grid,
			ContinuousSpace<Object> space, Behaviour behaviour) {
		super(mass, context, grid, space, behaviour);
		this.killers = new OsteoclastKiller[9];
		this.positions = new NdPoint[this.killers.length];
		this.discPositions = new GridPoint[this.killers.length];

		for (int i = 0; i < this.killers.length; i++) {
			this.killers[i] = new OsteoclastKiller(1, context, grid, space, null, this);
			context.add(this.killers[i]);
		}
		initKillerPositions();
		
		try {
			// Behaviour[] bs = new Sequence[8];
			// for (int i = 0; i < 4; i++)
			// bs[i] = new Sequence(new Bind(new InChannel("rank", this,
			// new Surface(i), 0.5)), new Split(new InChannel("rank",
			// this, new Surface(i), 1)));
			// for (int i = 4; i < 8; i++)
			// bs[i] = new Sequence(new Bind(new InChannel("rank'", this,
			// new Surface(i - 4), 1)), new Split(new InChannel(
			// "rank'", this, new Surface(i - 4), 1)));
			Behaviour[] bs = new Sequence[4];
			for (int i = 0; i < 4; i++)
				bs[i] = new Sequence(new Bind(new InChannel("rank'", this,
						new Surface(i), 1)), new Split(new InChannel("rank'",
						this, new Surface(i), 1)));
//			final Osteoclast oThis = this;
			this.setBehaviour(new Sequence(new Choice(bs), new ActivateKiller(
					this, this.killers), new Recursion(
			/* new Kill, */new Resorption(this))));
		} catch (ShapeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (BehaviourException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	private void initKillerPositions(){
		int n=this.positions.length;
        int levels=(int) Math.sqrt(n);
//        int rems=(n- (int) Math.pow(levels,2));
        int i=-levels/2;
        int imax=levels+i;
        int j=i;
        int jmax=imax;
        int index=0;
        while(i<imax){
        	while(j<jmax){
				this.discPositions[index] = new GridPoint(i, j);
				this.positions[index] = new NdPoint(i, j);
        		index++;
        		j++;
        	}
        	i++;
        }
        if(levels%2==0){
        	j=-levels/2;
        	jmax=levels+j+1;
        	while(index<n&&j<jmax){
				this.discPositions[index] = new GridPoint(imax, j);
				this.positions[index] = new NdPoint(imax, j);
        		j++;
        		index++;
        	}
        	i=-levels/2;
        	while(index<n){
				this.discPositions[index] = new GridPoint(i, jmax-1);
				this.positions[index] = new NdPoint(i, jmax-1);
        		i++;
        		index++;
        	}
        }
        else{
        	j=-levels/2 - 1;
        	i=-levels/2 - 1;
        	while(index<n&&j<jmax){
				this.discPositions[index] = new GridPoint(i, j);
				this.positions[index] = new NdPoint(i, j);
        		j++;
        		index++;
        	}
        	i++;
        	j=-levels/2 - 1;
        	while(index<n){
				this.discPositions[index] = new GridPoint(i, j);
				this.positions[index] = new NdPoint(i, j);
        		i++;
        		index++;
        	}
        }
	}
	
	public boolean isActive() {
		return isActive;
	}

	public void setActive(boolean isActive) {
		this.isActive = isActive;
	}

	public boolean hasreached() {
		return hasreached;
	}

	public void setReached(boolean hasreached) {
		this.hasreached = hasreached;
	}

	@Override
	public void updateVelocity() {
		for (int i = 0; i < this.killers.length; i++) {
			this.killers[i].setPosition(Utils.abs(Utils.sumPoints(this.getPosition(),
					this.positions[i])));
			this.killers[i].setDiscretePosition(Utils.abs(Utils.sumPoints(
					this.getDiscretePosition(), this.discPositions[i])));
		}
		super.updateVelocity();
		for (ShapeKiller killer : this.killers) {
			killer.setVelocity(this.getVelocity());
			killer.setDiscreteVelocity(this.getDiscreteVelocity());
			killer.updateVelocity();
		}
	}

	// @Override
	// public boolean moveTo(GridPoint newPosition, boolean ignoreComplexShape)
	// {
	// return super.moveTo(newPosition, ignoreComplexShape)
	// /* && this.killer.moveTo(newPosition, ignoreComplexShape) */;
	// }

	@Override
	public void setDiscretePosition(GridPoint discretePosition) {
		super.setDiscretePosition(discretePosition);
		for (int i = 0; i < this.killers.length; i++)
			this.killers[i].setDiscretePosition(Utils.abs(Utils.sumPoints(
					discretePosition, this.discPositions[i])));
	}

	@Override
	public void setPosition(NdPoint position) {
		super.setPosition(position);
		for (int i = 0; i < this.killers.length; i++)
			this.killers[i].setPosition(Utils.abs(Utils.sumPoints(position,
					this.positions[i])));
	}

	@Override
	public void setDiscreteVelocity(GridPoint discreteVelocity) {
		super.setDiscreteVelocity(discreteVelocity);
		for (ShapeKiller killer : this.killers)
			killer.setDiscreteVelocity(discreteVelocity);
	}

	@Override
	public void setVelocity(NdPoint velocity) {
		super.setVelocity(velocity);
		for (ShapeKiller killer : this.killers)
			killer.setVelocity(velocity);
	}

	@Override
	public int isOsteoclast() {
		return 1;
	}

}
